"conv.bayes" <-
function(y, m, K, K.pred=NULL, iter=2000, ay=1, by=0.005, ax=1, bx=0.005)
{
	#lmvt <- library(mvtnorm, logical.return=TRUE)
	lmvt <- FALSE
	n <- length(y)

	# intercept
	K <- cbind(rep(1,n), K);
	if(!is.null(K.pred)) K.pred <- cbind(rep(1,dim(K.pred)[1]), K.pred)
	m <- m+1

	# make the matrix W
	W <- diag(1, m);

	# arrays to hold the MCMC realizations
	# z = Kx 
	# y = z + e

	xout <- matrix(NA,ncol=m,nrow=iter)
	zout <- matrix(NA,ncol=n,nrow=iter)
	if(!is.null(K.pred)) zout.pred <- matrix(NA, ncol=dim(K.pred)[1], nrow=iter)
	else zout.pred <- NULL
	lamy <- rep(NA,iter)
	lamx <- rep(NA,iter)

	# give initial values to parameters
	lamy[1] <- 1; lamx[1] <- .2; xout[1,] <- 0; zout[1,] <- 0;

	# run the mcmc
	for(k in 2:iter){
  		prec <- lamy[k-1]*t(K)%*%K+lamx[k-1]*W
  		vmat <- solve(prec)
  		vmat <- .5*(vmat + t(vmat))
  		mean <- as.vector(lamy[k-1]*vmat%*%t(K)%*%as.vector(y))
  		if(! lmvt) xout[k,] <- rmultnorm(1,mean,vmat)
		else xout[k,] = rmvnorm(1,mean,vmat)
  		zout[k,] <- as.vector(K%*%xout[k,])
  		if(!is.null(K.pred)) zout.pred[k,] <- as.vector(K.pred%*%xout[k,])

  		rss <- t(y-zout[k,])%*%(y-zout[k,])
  		lamy[k] <- rgamma(1,ay+n/2)/(by+.5*rss)
		 
  		xss <- t(xout[k,])%*%W%*%xout[k,]
  		lamx[k] <- rgamma(1,ax+m/2)/(bx+.5*xss)
	}

	return(list(method="bayes", a=list(x=xout, z=zout, z.pred=zout.pred, lamy=lamy, lamx=lamx), df=list(K=K, y=y, K.pred=K.pred)))
}

