"conv.bayes.multi" <-
function(y, w, K, K.pred=NULL, iter=2000, ay=1, by=0.005, ax=1, bx=0.005)
{
	#lmvt <- library(mvtnorm, logical.return=TRUE)
	lmvt <- FALSE
	
	n <- length(y)
	m <- length(w);
	mw <- list(); W <- list(); lamx <- list(); xout <- list();
	
	for(i in 1:m) {
		mws <- dim(w[[i]])[1];
		if(is.null(mws)) { mws <- length(w[[i]]) }
		mw[[i]] <- mws;
	}

	# arrays to hold the MCMC realizations
	# z = K1*x1 + ... + Kn*xn

	# intercept and frist W matrix, etc
	# give initial values to parameters
	K[[1]] <- cbind(rep(1,n), K[[1]])
	if(!is.null(K.pred)) K.pred[[1]] <- cbind(rep(1,dim(K.pred[[1]])[1]), K.pred[[1]])
	W[[1]] <- diag(1,mw[[1]]+1);
	xout[[1]] <- matrix(NA,ncol=mw[[1]]+1,nrow=iter)
	xout[[1]][1] <- 0
	lamx[[1]] <- rep(NA,iter)
	lamx[[1]][1] <- 0.2

	# make the matrix W
	# give initial values to parameters
	for(i in 2:m) { 
		W[[i]] <- diag(1, mw[[i]]);
		xout[[i]] <- matrix(NA,ncol=mw[[i]],nrow=iter)
		xout[[i]][1] <- 0
		lamx[[i]] <- rep(NA,iter)
		lamx[[i]][1] <- 0.2
	}

	# y = z + e
	# give initial values to parameters
	zout <- matrix(NA,ncol=n,nrow=iter)
	zout[1,] <- 0
	if(!is.null(K.pred)) {
		zout.pred <- matrix(NA, ncol=dim(K.pred[[1]])[1], nrow=iter)
		zout.pred[1,] <- 0
	} else zout.pred <- NULL
	lamy <- rep(NA,iter)
	lamy[1] <- 1

	# run the mcmc
	for(k in 2:iter){
		zout[k,] <- rep(0,n)
		if(!is.null(K.pred)) zout.pred[k,] <- rep(0, dim(K.pred[[1]])[1])

		# for each resolution
		for(i in 1:m) {

			# residual responses
			yi <- as.vector(y) - zout[k,]

			# sample residual x process
  			prec <- lamy[k-1]*t(K[[i]])%*%K[[i]]+lamx[[i]][k-1]*W[[i]]
  			vmat <- solve(prec)
  			vmat <- .5*(vmat + t(vmat))
  			mean <- as.vector(lamy[k-1]*vmat%*%t(K[[i]])%*%yi)
  			if(! lmvt) xout[[i]][k,] <- rmultnorm(1,mean,vmat)
			else xout[[i]][k,] = rmvnorm(1,mean,vmat)

			# updated predictions
			zi <- as.vector(K[[i]]%*%xout[[i]][k,])
  			zout[k,] <- zout[k,] + zi
  			if(!is.null(K.pred)) {
				zi <- as.vector(K.pred[[i]]%*%xout[[i]][k,])
  				zout.pred[k,] <- zout.pred[k,] + zi
			}

			# resolution error
  			xss <- t(xout[[i]][k,])%*%W[[i]]%*%xout[[i]][k,]
  			lamx[[i]][k] <- rgamma(1,ax+mw[[i]]/2)/(bx+.5*xss)
		}

		# additive error
  		rss <- t(y-zout[k,])%*%(y-zout[k,])
  		lamy[k] <- rgamma(1,ay+n/2)/(by+.5*rss)
		 
	}

	return(list(method="bayes", a=list(x=xout, z=zout, z.pred=zout.pred, lamy=lamy, lamx=lamx), 
		df=list(K=K, y=y, K.pred=K.pred)))
}

