"conv.gp.mc.1d" <-
function(data.f, data.c, pdata.f=NULL, pdata.c=NULL, w, sdkern=2, iter=2000, ay=1, by=0.005, ax=1, bx=0.005)
{
	#lmvt <- library(mvtnorm, logical.return=TRUE)
	lmvt <- FALSE
	
	# input data
	sf <- data.f$s; yf <- data.f$y;
	sc <- data.c$s; yc <- data.c$y;
	m <- length(w)+1  # +1 for the intercept
        nf <- length(yf); nc <- length(yc);
	
	# create the K and W matrices 
	Kf <- cbind(rep(1, nf), new.K.1d(sf, w, sdkern))
	Kc <- cbind(rep(1, nc), new.K.1d(sc, w, sdkern))
	if(!is.null(pdata.f)) Kf.pred <- cbind(rep(1, length(pdata.f)), new.K.1d(pdata.f, w, sdkern))
	if(!is.null(pdata.c)) Kc.pred <- cbind(rep(1, length(pdata.c)), new.K.1d(pdata.c, w, sdkern))
	W <- diag(1, m);

	# arrays to hold the MCMC realizations
	# z = Kx 
	# y = z + e
	xfout <- matrix(NA,ncol=m,nrow=iter); zfout <- matrix(NA,ncol=nf,nrow=iter)
	xcout <- matrix(NA,ncol=m,nrow=iter); zcout <- matrix(NA,ncol=nc,nrow=iter)
	lamyf <- rep(NA,iter); lamxf <- rep(NA,iter)
	lamyc <- rep(NA,iter); lamxc <- rep(NA,iter)

	# similar arrays for predictive locations
	if(!is.null(Kf.pred)) {
		zfout.pred <- matrix(NA, ncol=dim(Kf.pred)[1], nrow=iter)
		zfout.pred[1,] <- 0
	} else zfout.pred <- NULL
	if(!is.null(Kc.pred)) {
		zcout.pred <- matrix(NA, ncol=dim(Kc.pred)[1], nrow=iter)
		zcout.pred[1,] <- 0
	} else zfout.pred <- NULL


	# give initial values to parameters
	lamyf[1] <- 1; lamxf[1] <- .2; xfout[1,] <- 0; zfout[1,] <- 0;
	lamyc[1] <- 1; lamxc[1] <- .2; xcout[1,] <- 0; zcout[1,] <- 0;

	# run the mcmc
	for(i in 2:iter){
          if(i%%10){  # do regular MCMC except swap every 10th iteration
	  #print(paste("    ", i, "fine"))
            # regular fine scale updates
            prec <- lamyf[i-1]*t(Kf)%*%Kf+lamxf[i-1]*W
            vmat <- solve(prec)
            vmat <- .5*(vmat + t(vmat))
            mean <- as.vector(lamyf[i-1]*vmat%*%t(Kf)%*%as.vector(yf))
            if(!lmvt) xfout[i,] <- rmultnorm(1,mean,vmat)
            else xfout[i,] = rmvnorm(1,mean,vmat)
            zfout[i,] <- as.vector(Kf%*%xfout[i,])
            if(!is.null(Kf.pred)) zfout.pred[i,] <- as.vector(Kf.pred%*%xfout[i,])
            rssf <- t(yf-zfout[i,])%*%(yf-zfout[i,])
            lamyf[i] <- rgamma(1,ay+nf/2)/(by+.5*rssf)
            xssf <- t(xfout[i,])%*%W%*%xfout[i,]
            lamxf[i] <- rgamma(1,ax+m/2)/(bx+.5*xssf)

            # regular coarse scale updates
	  #print(paste("    ", i, "coarse"))
            prec <- lamyc[i-1]*t(Kc)%*%Kc+lamxc[i-1]*W
            vmat <- solve(prec)
            vmat <- .5*(vmat + t(vmat))
            mean <- as.vector(lamyc[i-1]*vmat%*%t(Kc)%*%as.vector(yc))
            if(!lmvt) xcout[i,] <- rmultnorm(1,mean,vmat)
            else xcout[i,] = rmvnorm(1,mean,vmat)
            zcout[i,] <- as.vector(Kc%*%xcout[i,])
            if(!is.null(Kc.pred)) zcout.pred[i,] <- as.vector(Kc.pred%*%xcout[i,])
            rssc <- t(yc-zcout[i,])%*%(yc-zcout[i,])
            lamyc[i] <- rgamma(1,ay+nc/2)/(by+.5*rssc)
            xssc <- t(xcout[i,])%*%W%*%xcout[i,]
            lamxc[i] <- rgamma(1,ax+m/2)/(bx+.5*xssc)
          }

          else {  # Metropolis-coupled swapping update
	  #print(i)
            xfout[i,] <- xcout[i-1,]
            xcout[i,] <- xfout[i-1,]
            zfout[i,] <- as.vector(Kf%*%xfout[i,])
            if(!is.null(Kf.pred)) zfout.pred[i,] <- as.vector(Kf.pred%*%xfout[i,])
            zcout[i,] <- as.vector(Kc%*%xcout[i,])
            if(!is.null(Kc.pred)) zcout.pred[i,] <- as.vector(Kc.pred%*%xcout[i,])
            rssf <- t(yf-zfout[i,])%*%(yf-zfout[i,])
            lamyf[i] <- rgamma(1,ay+nf/2)/(by+.5*rssf)
            rssc <- t(yc-zcout[i,])%*%(yc-zcout[i,])
            lamyc[i] <- rgamma(1,ay+nc/2)/(by+.5*rssc)
            xssf <- t(xfout[i,])%*%W%*%xfout[i,]
            lamxf[i] <- rgamma(1,ax+m/2)/(bx+.5*xssf)
            xssc <- t(xcout[i,])%*%W%*%xcout[i,]
            lamxc[i] <- rgamma(1,ax+m/2)/(bx+.5*xssc)
            # compute M-H acceptance probability
	    # in this case it is 1.0, so nothing to do
          }
	}

	return(list(method="met.coup", a=list(x=xfout, xc=xcout, z=zfout, zc=zcout, z.pred=zfout.pred, zc.pred=zcout.pred, 
		lamy=lamyf, lamyc=lamyc, lamx=lamxf, lamxc=lamxc), 
		df=list(K=Kf, Kc=Kc, K.pred=Kf.pred, Kc.pred=Kc.pred, y=yf, yc=yc), s=sf,sc=sc,s.pred=pdata.f, sc.pred=pdata.c,w=w,sdkern=sdkern))
}

