\name{conv.gp}
\alias{conv.gp.1d}
\alias{conv.gp.2d}
\alias{conv.gp.multi.1d}
\alias{conv.gp.multi.2d}
\alias{conv.gp.mc.1d}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Gaussian process modeling via Spatial Convolutions }
\description{
  Functions for maximum likelihood (ML) and Bayesian estimation and
  prediction for the Spatial Convolution implementation of Gaussian
  process spatial/regression models with fixed and multi-fidelity
  smoothing kernel configurations -- for 1-d and 2d- data.
}
\usage{
conv.gp.1d(data, pdata = NULL, w, sdkern = 2, method = "bayes",
           iter = 2000, ay = 1, by = 0.005, ax = 1, bx = 0.005)
conv.gp.2d(data, pdata = NULL, w, sdkern, method = "bayes",
           iter = 2000, ay = 1, by = 0.005, ax = 1, bx = 0.005)
conv.gp.multi.1d(data, pdata = NULL, w, sdkern, method = "bayes",
                 iter = 2000, ay = 1, by = 0.005, ax = 1, bx = 0.005)
conv.gp.multi.2d(data, pdata = NULL, w, sdkern, method = "bayes",
                 iter = 2000, ay = 1, by = 0.005, ax = 1, bx = 0.005)
conv.gp.mc.1d(data.f, data.c, pdata.f = NULL, pdata.c = NULL, w,
              sdkern = 2, iter = 2000, ay = 1, by = 0.005, ax = 1,
              bx = 0.005)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{ \code{data.frame} or \code{list} object containing fields
    \code{s} containing (spatial) input locations (vector for 1-d, 2-d
    matrix for 2-d) and \code{y} of the same number of responses. MC
    method(s) do not use this argument, see next two arguments.... }
  \item{data.f}{ (\dQuote{mc}-- Metropolis Coupled-- 1-d only) \code{data} for
    fine level process }
  \item{data.c}{ (\dQuote{mc}-- Metropolis Coupled-- 1-d only) \code{data} for
    coarse level process }
  \item{pdata}{ Optional vector (1-d) or \code{matrix} (or \code{data.frame}) of
    spatial locations for for (posterior) predictive inference.  MC
    method(s) do not use this argument, see next two arguments }
  \item{pdata.f}{ (\dQuote{mc}-- Metropolis Coupled-- 1-d only), \code{pdata}
    locations for fine level process, or \code{NULL} }
  \item{pdata.c}{ (\dQuote{mc}-- Metropolis Coupled-- 1-d only), \code{pdata}
    for coarse level process, or \code{NULL} }
  \item{w}{ Vector (2-d) or \code{matrix} (or \code{data.frame}) of
    convolution kernel locations.  For \dQuote{multi} functions,
    \code{w} must be a list of vectors or matrices of length equal to
    the specified resolution }
  \item{sdkern}{ Positive scalar (or vector of \code{length(w) ==
      length(sdkern)} of widths of convolution kernels }
  \item{method}{ If \dQuote{bayes} (default) then Bayesian inference is
    used, otherwise REML-style maximum likelihood is used }
  \item{iter}{ Number of MCMC samples for Bayesian inference.  Ignored
    if \code{method != "bayes"}}
  \item{ay}{ Gamma (alpha) prior parameter for lamy, ignored for
    \code{method != "bayes"} }
  \item{by}{ Gamma (beta) prior parameter for lamy, ignored for
    \code{method != "bayes"} }
  \item{ax}{ Gamma (alpha) prior parameter for lamx, ignored for
    \code{method != "bayes"} }
  \item{bx}{ Gamma (beta) prior parameter for lamx, ignored for
    \code{method != "bayes"} }
}
%\details{
   %~~ If necessary, more details than the description above ~~
%}
\value{
  Output (return value) is a list containing (at least) the following entries
  
  \item{method }{ A copy of the \code{method} input parameter }
  \item{a }{ For Bayesian models: A list containing the following
    entries, which pertain to MCMC samples  for the
    following parameters:
    \itemize{
      \item{x }{The white noise \code{X} process at the kernel locations
	\code{w} }
      \item{z}{ Posterior predictive samples at the data locations
	\code{s}}
      \item{z.pred}{ Posterior predictive samples at the new predictive
	locations \code{pdata} }
      \item{lamy}{ Samples for the lamy parameter}
      \item{lamx}{ Samples for the lamx parameter}
      }
      Non Bayesian models contain the results from the \code{\link[nlme]{lme}}
      command, along with the \code{z} and \code{z.pred} items from above
  }
  \item{df }{ For Bayesian models: A list containing the following
    entries, which pertain to the Kernels:
    \itemize{
      \item{K}{ The computed kernels at locations \code{w} for the inputs
	\code{data$s}, based on widths \code{sdkern}}
      \item{y}{ A copy of the input responses \code{data$y} }
      \item{K.pred}{ The computed kernels at the locations \code{w} for
	the inputs \code{data$s} based on widths \code{sdkern}}
      }
      Non Bayesian models contain the results a Kernel vector or
      \code{matrix} foe each of the spatial input locations \code{data$s}
      in addition to the \code{y} from above
  }
  \item{s }{ A copy of the input spatial locations \code{data$s} }
  \item{w }{ A copy of the input kernel locations \code{w} }
  \item{sdkern }{ A copy of the input kernel widths \code{sdkern}  }
  \item{s.pred}{ A copy of the input predictive locations \code{pdata} }
}

\references{
Higdon, D. (2002).
\emph{Space and space-time modeling using process convolutions.}
Quantitative Methods for Current Environmental Issues (pp.37--56).
London: Springer-Verlag.

Ferreira, M. A., Higdon, D., Lee, H. K., and West, M. (2005).
\emph{Multi-scale random field models.}
DME--UFRG Technical report, \emph{submitted};
  \url{http://www.dme.ufrj.br/~marco/msmrf.pdf}
}
 
\author{ Robert B. Gramacy \email{rbgramacy@ams.ucsc.edu} }

%\note{ ~~further notes~~ 
% ~Make other sections like Warning with \section{Warning }{....} ~
%}

\seealso{ \code{\link{plot.conv}} }

\examples{
# -------------------------
# ONE dimensional synthetic  data
# --------------------------------

# get the synthetic data
data <- data.1d()
pdata <- seq(1,10,length=100)

# -----------------------
# SINGLE Resolution model
# -----------------------

# centers of the kernels (must be a matrix)
w <- as.matrix(seq(1-2,10+2,length=7))

# prepare the 1-d ML convolution model
data.conv.gp <- conv.gp.1d(data, pdata=pdata, w=w, sdkern=2)

# plot the results
plot(data.conv.gp, main="1-d Single Resolution")


\dontrun{
# ----------------------
# MULTI Resolution model
# ----------------------

# kernels: number of support points at each scale
m1 <- c(7,14,28) 

# kernel centers: list to hold the support locations
w1 <- list()  # (must be a list)
for(i in 1:3) w1[[i]] <- seq(1-2,10+2,length=m1[i])

# kernels: width by scale
sdkern <- c(2,1,.5)

# estimate the 1-d ML convolution model
data.conv.gp.multi <- conv.gp.multi.1d(data, pdata=seq(1,10,length=100), w=w1, sdkern=sdkern)

# plot the results
plot(data.conv.gp.multi, main="1-d Multiple Resolution")
}


# ___________________________________________________________________________
# ___________________________________________________________________________
#

# ---------------------------
# TWO dimensional ozone data
# ---------------------------

# get the 2d ozone data
ozone <- data.ozone()
x <- ozone$s[,1]; y <- ozone$s[,2]
pdata <- as.matrix(grid.2d(x,y,20,20))


# -----------------------
# SINGLE Resolution model
# -----------------------

# locations of support points w (must be a matrix)
wrows <- 6; wcols <- 8
w <- as.matrix(grid.2d(x,y,wrows,wcols))

# width of kernel
sdkern <- 2*diag(1,2,2)

# prepare the 2-d ML convolution model
ozone.conv.gp <- conv.gp.2d(ozone, pdata=pdata, w=w, sdkern=sdkern)

# plot the results
plot(ozone.conv.gp, main="2-d Ozone")


\dontrun{
# -------------------------
# MULTIPLE Resolution model
# -------------------------

# make support locations w (must be a list)
w <- list( sparce=grid.2d(x,y,4,5), med=grid.2d(x,y,6,8), fine=grid.2d(x,y,9,12))

# width of the kernels
sdkern <- list( sparce=2*diag(1,2,2), med=1*diag(1,2,2), fine=0.5*diag(1,2,2))

# estimate the 2-d ML convolution model
ozone.conv.gp.multi <- conv.gp.multi.2d(ozone, pdata=pdata, w=w, sdkern=sdkern)

# plot the results
plot(ozone.conv.gp.multi, main="2-d Ozone Multi")
}
}
\keyword{ models }% at least one, from doc/KEYWORDS
\keyword{ spatial }% __ONLY ONE__ keyword per line
