\name{hrm}
\alias{hrm}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Hidden Resolution Time Series Model }
\description{
  Code to fit a Bayesian Hidden Resolution Time Series Model
  using Markov chain Monte Carlo (MCMC)
}
\usage{
hrm(data, m = 12, Gbig = 5000, nys = 5, harms = c(6, 3, 2))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{ Vector of time series input data, representing the fine
    level process }
  \item{m}{ Integer, scalar of hierarchical (seasonal) process, e.g.,
    months (12); number of fine-scale observations in the noisy
    block-average process, e.g., annual averages }
  \item{Gbig}{ Integer, maximum number of MCMC rounds } 
  \item{nys}{ Integer, number of coarse level times to save for posterior
    use by the forecast program }
  \item{harms}{ Vector of harmonics for seasonal component of the model }
}
\details{
  Inference via MCMC implemented by Gibbs sampling
}
\value{
  Output is a \code{list} object with the following entries:
  
  \item{ data }{ A copy of the input data vector }
  \item{ nx }{ length of the input data vector, i.e., \code{length(data)} }
  \item{ m }{ A copy of the input parameter \code{m}}
  \item{ Gbig }{ A copy of the MCMC rounds input parameter}
  \item{ nys }{ A copy of the input parameter, \code{nys} }
  \item{ nh }{ Length of the input vector \code{harms}, i.e., \code{length(harms)}}
  \item{ harms }{ A copy of the input vector of harmonics}
  \item{ lambda }{ MCMC samples from the posterior distribution for
  lambda, representing the relative increase in uncertainty at the
  course level due to lack of agreement at the fine level }
  \item{ ysample }{ MCMC samples from \code{nys} course level times for
  use by the forecast program }
  \item{ theta }{ \code{data.frame} containing MCMC samples from the
    posterior distribution for the model parameters phiy, sigmay2, phix,
    sigmax2, tau, and beta}
}

\references{ Marco A. R. Ferreira, Mike West, Herbert K. H. Lee and
	David Higdon (2006), \emph{Multi-scale and hidden resolution time
	series models.} Bayesian Analysis, to appear.  }

\author{ Robert B. Gramacy \email{rbgramacy@ams.ucsc.edu} and Marco Ferreira
\email{marco@im.ufrj.br}}

%\note{ ~~further notes~~ }

\seealso{ \code{\link{plot.hrm}}, \code{\link{hrm.fcast}}
  \code{\link{plot.hrm.fcast}}}

\examples{
# ------------------------------------------------------------
# use the full time series to fit the model for the RIVER data
# ------------------------------------------------------------

data(river)

# fit the model  (MCMC)
river.hrm <- hrm(river, harms=c(6,3,2))

# view the posterior of the parameters
plot(river.hrm)
# follow by three empty/comment lines for R CMD CHECK

# forecast (MCMC), three years ahead
river.hrm <- hrm.fcast(river.hrm, window=3)

# plots of forecasts
plot.hrm.fcast(river.hrm, start=1991)
# follow by three empty/comment lines for R CMD CHECK


\dontrun{
# ---------------------------------------------------------------
# fit the model for the RIVER data (leaving off last three years)
# ---------------------------------------------------------------

# fit the model  (MCMC)
river.hrm <- hrm(river[1:(length(river)-3*12)], harms=c(6,3,2))

# view the posterior of the parameters
plot(river.hrm)
# follow by three empty/comment lines from R CMD CHECK


# forecast (MCMC), three years ahead
river.hrm <- hrm.fcast(river.hrm, window=3)

# plots of forecasts, follow by three empty/comment lines
plot.hrm.fcast(river.hrm, start=1988,
         truth=river[(length(river)-3*12+1):length(river)])
}

# --------------------------------------------------------------
# use the full time series to fit the model for the SUDESTE data
# --------------------------------------------------------------

data(sudeste)

# fit the model  (MCMC)
sudeste.hrm <- hrm(sudeste, harms=c(6,3))

# view the posterior of the parameters
plot(sudeste.hrm)
# follow by three empty/comment lines for R CMD CHECK


# forecast (MCMC), three years ahead
sudeste.hrm <- hrm.fcast(sudeste.hrm, window=3)

# plots of forecasts
plot.hrm.fcast(sudeste.hrm, start=2001)


\dontrun{
# -----------------------------------------------------------------
# fit the model for the SUDESTE data (leaving off last three years)
# -----------------------------------------------------------------

# fit the model  (MCMC)
sudeste.hrm <- hrm(sudeste[1:(length(sudeste)-3*12)], harms=c(6,3))

# view the posterior of the parameters
plot(sudeste.hrm)
# follow by three empty/comment lines for R CMD CHECK


# forecast (MCMC), three years ahead
sudeste.hrm <- hrm.fcast(sudeste.hrm, window=3)

# plots of forecasts, follow by three empty/comment lines
plot.hrm.fcast(sudeste.hrm, start=1998,
         truth=sudeste[(length(sudeste)-3*12+1):length(sudeste)])
}


}
\keyword{ models }% at least one, from doc/KEYWORDS
\keyword{ ts }% __ONLY ONE__ keyword per line
