#include <stdlib.h>	
#include <math.h>
#include <stdio.h>

void compAVAt(double phix, double sigmax2, int m, int ny , double *AVAt)
{
  /* 
    Matrices are in FORTRAN format!!! (Note that here AVAt is symmetric)
    Computes A * Vx * A'  in the particular case 
       A = (1/m) Kroneckerproduct(I_{n_y}, 1_m)
    and
       Vx = Covariance matrix of a AR(1) process

     Note that A * Vx * A' is a symmetric Toeplitz matrix in this case.
       */

double aux;
int i, j;


 /* Computing the main diagonal ... */
 AVAt[0] = (sigmax2*(m-m*phix*phix-2*phix+2*pow(phix,(double)(m+1))))
             /((1-phix*phix)*pow(m*(1-phix),2.0));
 for(i=1;i<ny;i++)  AVAt[i*ny+i] = AVAt[0];

 /* Computing the first line ... */
 aux = (sigmax2*(phix-2*pow(phix,(double)(m+1))+pow(phix,(double)(2*m+1))))
             /((1-phix*phix)*pow(m*(1-phix),2.0));
 i=0;
 for(j=i+1;j<ny;j++) AVAt[i*ny+j] = aux*pow(phix,(double)((abs(i-j)-1)*m));

 /* Computing the superior triangle ... */
 for(i=1;i<ny;i++)  
  for(j=i+1;j<ny;j++)  
   AVAt[i*ny+j] = AVAt[(i-1)*ny+j-1];

 /* Computing the inferior triangle ... */
 for(i=0;i<ny;i++)  
  for(j=i+1;j<ny;j++)  
   AVAt[j*ny+i] = AVAt[i*ny+j];
} /* End of compAVAt */

/*
int main(void)
{
double *AVAt, phix, sigmax2;
int m, nx, ny, i, j;

 AVAt = (double *) malloc(ny * ny * sizeof(double));
 phix = 0.9;
 sigmax2 = 1.0;
 m  =  12;
 nx = 600;
 ny =  50;

for(i=0;i<10000;i++){ 
 compAVAt(phix, sigmax2, m, ny , AVAt);
}

 for(i=0;i<6;i++){  
  for(j=0;j<6;j++) printf("%10.4f ",AVAt[i*ny+j]);
  printf("\n");
 }

 free(AVAt);
}
*/


