#include "setup.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "rand.h"
#include "rmvnormd.h"
#include "matrix.h"
#include "tipos.h"

int nregr;

 /* Prior parameters  */
 double mphiy = 0.0;
 double sphiy = 1000.0;
 double nsy2  = 0.0001;
 double nssy2 = 0.0001;
 double mphix = 0.0;
 double sphix = 1000.0;
 double nsx2  = 0.0001;
 double nssx2 = 0.0001;
 double ntau  = 0.0001;
 double nstau = 0.0001;
 double nlambda = 12.0;
 double nslambda = 5.0;

 /* Metropolis "parameters" */
 double fatphix = 100.0; /* factor to decrease the variance of the
			  proposal for phix */
 double ftau = 1.0;   /* factor to decrease the df for tau */
 double fsigmax2 = 0.1;   /* factor to decrease the df for sigmax2 */
 double deltalambda = 1.25; /* Must be > 1.0 */
 double deltaphix = 0.1;
 double deltasx2 = 1.15; /* Must be > 1.0 */
 double delta = 0.5;

 double CbetaI[] = {.00001,    0.0,     0.0,     0.0,     0.0,     0.0,  
                     0.0,   .00001,     0.0,     0.0,     0.0,     0.0,
                     0.0,      0.0,  .00001,     0.0,     0.0,     0.0, 
                     0.0,      0.0,     0.0,   .00001,    0.0,     0.0,
                     0.0,      0.0,     0.0,     0.0,   .00001,    0.0,
                     0.0,      0.0,     0.0,     0.0,     0.0,   .00001};
 /* We assume that mbeta = 0 */

 /*    Matrices are in FORTRAN format!!!   */


void cholD(int nx, double *D, double *Lauxx)
{
int info;
double alpha;

  /* Computing Lauxx = Cholesky(D) */
  alpha = 1.0;
  dgemt("N",&nx,&nx,&alpha,D,&nx,Lauxx,&nx); 
  dpotrf("L",&nx,Lauxx,&nx,&info);
  return;
} /* End of CholD */


double compexp(int ny, double *ylevel, double *Lauxx)
{
/* computing exponent = -0.5 y' W^(-1) y  
   where Lauxx = cholesky(W) */

  double exponent, *vauxx;
  int inc;/*, i;*/

  vauxx = (double *)malloc((unsigned) (ny)*sizeof(double));

  inc = 1;
  dcopy(&ny,ylevel,&inc,vauxx,&inc);

  dtrsv("L","N","N",&ny,Lauxx,&ny,vauxx,&inc);
  exponent = -0.5 * dnrsq(&ny,vauxx,&inc);

  free(vauxx);

  return exponent;
} /* End of compexp */

double logdet(int ny, double *Lauxx)
{
  /* Computing logdet = log(det(Lauxx)), 
     where Lauxx is a ny by ny lower triangular matrix. */

  double logdete;  
  int i;

  logdete=0.0;
  for(i=0;i<ny;i++){
    if (Lauxx[i*ny+i]>0) logdete += log(Lauxx[i*ny+i]);
    else printf("i=%d,  Lauxx[i,i]=%f  ",i,Lauxx[i*ny+i]);
  }

  return logdete;
} /* End of logdet */


double compygt(PARAM *t, int m, int nx, int ny, 
               double *ylevel)
{
double *W, *Lauxx, logpygt, exponent;
/*int i,j;*/
  /* Computing    log p(y) */ 
  W = (double *)malloc((unsigned) (ny*ny)*sizeof(double));
  Lauxx = (double *)malloc((unsigned) (ny*ny)*sizeof(double));

  compW(t,m,nx,ny,W);
  cholD(ny,W,Lauxx);
  exponent = compexp(ny,ylevel,Lauxx);
  logpygt = exponent - logdet(ny,Lauxx);

  free(W);
  free(Lauxx);  

  return logpygt;
} /* End of compygt */

void compqygx(PARAM *t, int m, int nx, int ny, double *Ax, double *my,
	      double *Vy)
{
  double *WI, *vetaux;
  int i, j, inc, auxi;
  double al, be, aux;
  /* Computing my = E_q[y | x] and Vy = V_q[y|x] */

  WI = (double *)malloc((unsigned) (ny*ny)*sizeof(double));
  vetaux = (double *)malloc((unsigned) (ny)*sizeof(double));
  
  compW(t,m,nx,ny,Vy);  inverse(ny,Vy,WI);
  for(j=0;j<ny;j++){
   auxi = j*ny;
   for(i=0;i<ny;i++) WI[i+auxi] = - WI[i+auxi];
  }
  aux = (1.0 / (*t).tau) + (1.0 / (*t).sigmay2);
  WI[0] += aux;  WI[(ny-1)+(ny-1)*ny] += aux;
  aux = (1.0 / (*t).tau) + ((1.0+(*t).phiy * (*t).phiy) / (*t).sigmay2);
  for(i=1;i<(ny-1);i++) WI[i+i*ny] += aux;
  aux = - (*t).phiy / (*t).sigmay2;
  for(i=1;i<ny;i++) WI[i-1+i*ny] += aux;
  for(i=1;i<ny;i++) WI[i+(i-1)*ny] += aux;
  inverse(ny,WI,Vy);
  for(i=0;i<ny;i++) vetaux[i] = Ax[i] / (*t).tau;
  al=1.0; be=0.0; inc=1;
  dgemv("N",&ny,&ny,&al,Vy,&ny,vetaux,&inc,&be,my,&inc);

  free(WI);
  free(vetaux);
}


double logpygx(PARAM *t, int m, int nx, int ny, double *ylevel, 
               double *Ax, double *dif2xy)
{
  double aux;
  int /*i,*/ j;
  /* Computing log p(y | x) */

  *dif2xy = 0.0;
  for(j=0;j<ny;j++) *dif2xy += (ylevel[j]-Ax[j])*(ylevel[j]-Ax[j]);
  aux = -0.5*ny*log((*t).tau) - 0.5*(*dif2xy)/(*t).tau;

  return aux;
}


double logqy(PARAM *t, int ny, double *ylevel, double *sumy2, double *sumyby)
{
  double aux;
  int i; /*j;*/
  /* Computing log q(y) */

  *sumy2 = 0.0;
  *sumyby = 0.0;
  for(i=1;i<ny;i++){
    *sumy2  += ylevel[i]*ylevel[i];
    *sumyby += ylevel[i]*ylevel[i-1];
  }

  aux = -0.5*(ny-1)*log((*t).sigmay2) - (0.5/(*t).sigmay2) *  
          (*sumy2 - 2 * (*t).phiy * *sumyby + (*t).phiy * 
            (*t).phiy * (*sumy2 + ylevel[0]*ylevel[0] 
              - ylevel[ny-1]*ylevel[ny-1]));

  return aux;
}


void compCAR1(double phi, double sigma2, int n, double *C)
{
  /* Computing C = Var(X_{1:n}), were X follows an AR(1) process with
     parameters phi and sigma2 */
int i, j;

  for(i=0;i<n;i++) for(j=i+1;j<n;j++) 
    C[i*n+j] = (sigma2 / (1-phi * phi))*pow(phi,(double)(abs(i-j)));
  for(i=0;i<n;i++) C[i*n+i] = (sigma2/(1-phi * phi));
  for(i=0;i<n;i++) for(j=i+1;j<n;j++) C[j*n+i] = C[i*n+j];

} /* End of compCAR1 */


void compQx( PARAM *t, int m, int nx, int ny, double *Qx)
{
  /* Computing 
         Qx = Vx - B (W - Qy) B'
     
     where: 
         W = A Vx A' + tau I
         B = Vx A'  W^(-1)
         Vx = Var( AR(1), phix, sigmax2, nx)
         Vy = Var( AR(1), phiy, sigmay2, ny)

     Note that the computation of AV and AVAt is set for the
     particular case when A = (1/m) Kroneckerproduct(I_{n_y}, 1_m) 
     and Vx = Covariance matrix of a AR(1) process.     */

  int i, /*j,*/ info;
  double alpha, beta, aux;
  double *B, *Bt, *W, *AV, *Vx, *Laux, *Qy;

  B  = (double *)malloc((unsigned) (nx*ny)*sizeof(double));
  Bt = (double *)malloc((unsigned) (ny*nx)*sizeof(double));
  AV = (double *)malloc((unsigned) (ny*nx)*sizeof(double));
  Vx = (double *)malloc((unsigned) (nx*nx)*sizeof(double));
  W = (double *)malloc((unsigned) (ny*ny)*sizeof(double));
  Laux = (double *)malloc((unsigned) (ny*ny)*sizeof(double));
  Qy = (double *)malloc((unsigned) (ny*ny)*sizeof(double));

  compAV((*t).phix,(*t).sigmax2,m,nx,ny,AV);
  compCAR1((*t).phiy,(*t).sigmay2,ny,Qy);

  /* Computing W = AVAT + tau * I */
  compAVAt((*t).phix,(*t).sigmax2,m,ny,W);   /* W = AVAt */
  aux=(*t).tau;
  for(i=0;i<ny;i++) W[i*ny+i]=W[i*ny+i]+aux;  /* W = AVAt + tau I*/

  /* Computing B transpose:  Bt = W^(-1) AV   */
  alpha = 1;
  dgemt("N",&ny,&ny,&alpha,W,&ny,Laux,&ny);  /* Laux = W */
  dgemt("N",&ny,&nx,&alpha,AV,&ny,Bt,&ny);  
  dposv("L",&ny,&nx,Laux,&ny,Bt,&ny,&info); 

  /* Computing B = (Bt)' */
  dgemt("T",&nx,&ny,&alpha,Bt,&ny,B,&nx);  

  /* Computing Qx = Vx - B (W - Qy) B' */
  alpha =  1.0;  beta  = -1.0;
  dgema("N","N",&ny,&ny,&alpha,W,&ny,&beta,Qy,&ny,W,&ny);  /* W = W - Qy */  
  alpha =  1.0;  beta  = 0.0;
  dgemm("N","N",&ny,&nx,&ny,&alpha,W,&ny,Bt,&ny,&beta,AV,&ny); /* AV = W B' */
  compCAR1((*t).phix,(*t).sigmax2,nx,Qx);  /*  Qx = Vx */
  alpha = -1.0;  beta  =  1.0;
  dgemm("N","N",&nx,&nx,&ny,&alpha,B,&nx,AV,&ny,&beta,Qx,&nx); /*Qx=Vx-B AV */

  free(B);
  free(Bt);
  free(AV);
  free(Vx);
  free(W);
  free(Laux);
  free(Qy);
 
  return;
} /*  End of compDandB */

#ifdef FALSE
double compqxgt(PARAM *t, int m, int nx, int ny, double *xlevel, double *zmatrix, double *logqxgt)
{
double *Qx, exponent, *mux, *xwm, aux;
int i, j, auxi;
/* Computing log q(x), that is, the log-likelihood based on the Hidden 
   Resolution Model */

  Qx = (double *)malloc((unsigned) (nx*nx)*sizeof(double));
  mux = (double *)malloc((unsigned) (nx)*sizeof(double));
  xwm = (double *)malloc((unsigned) (nx)*sizeof(double));

  /* Computing mux */
  for(i=0;i<nx;i++) mux[i]=0;
  for(j=0;j<nregr;j++){
    aux = (*t).beta[j]; auxi = j*nx;
    for(i=0;i<nx;i++) mux[i] += zmatrix[i+auxi] * aux;
  }

  /* Computing xwm = xlevel - mux */
  for(i=0;i<nx;i++) xwm[i]=xlevel[i]-mux[i];

  compQx(t,m,nx,ny,Qx);
  cholD(nx,Qx,Qx);
  exponent = compexp(nx,xwm,Qx);
  *logqxgt = exponent - logdet(nx,Qx);

  free(Qx);
  free(mux);
  free(xwm);

  return;
} /* End of compqxgt */
#endif

double compAVA11(double phix, double sigmax2, int m)
{
  double aux, auxpot;
  int i;
  /* Computing the element (1,1) of the matrix A V_x A' */

  if (phix>0) auxpot = exp((m+1)*log(phix));
  else if (phix==0) auxpot = 0.0;
  else { 
    auxpot = exp((m+1)*log(-phix));
    for(i=0;i<((m+1)%2);i++) auxpot *= (-1);
  }

  aux = sigmax2*(m-m*phix*phix-2*phix+2*auxpot)
    /(m*m*(1-phix)*(1-phix)*(1-phix*phix));

  return aux;
} /* End of compAVA11 */

void gibbs(PARAM *t, int m, int nx, int ny, double *xlevel, 
           double *ylevel, double *Ax, double sumx2, double sumxbx,
           double *zmatrix)
{

  /* Here is the Gibbs Sampler */

  double maux, vaux, naux, nsaux, aux, prop, logratio;
  double /*logfull, logfullold,*/ logpxgy, logpxgyold;
  int inc, info, auxi, i, j/*, ka*/;
  PARAM *tprop;
  double *W/*, *WI*/;
  double sumy2, sumyby, dif2xy;
  double sumy2prop, sumybyprop, dif2xyprop;
  double *ylevelprop, *vetaux, *vetaux2;
  double al, be;
  double lambda, /*lambdaprop,*/ ub, lb;
  double *mux, *ZtVI, *ZtVIZ, *mb, *Cb, *CbI, *auxvet, *xwm;
  char auxchar;

 /* Memory allocation */
  tprop= (PARAM *)malloc((unsigned) sizeof(PARAM));
  tprop->beta = (double*) malloc(sizeof(double) * nregr);
  W = (double *)malloc((unsigned) (ny*ny)*sizeof(double));
  vetaux = (double *)malloc((unsigned) (ny)*sizeof(double));
  vetaux2 = (double *)malloc((unsigned) (ny)*sizeof(double));
  ylevelprop = (double *)malloc((unsigned) (ny)*sizeof(double));
  mux    = (double *)malloc((unsigned) (nx)*sizeof(double));
  ZtVI   = (double *)malloc((unsigned) (nregr*nx)*sizeof(double));
  ZtVIZ  = (double *)malloc((unsigned) (nregr*nregr)*sizeof(double));
  mb     = (double *)malloc((unsigned) (nregr)*sizeof(double));
  auxvet = (double *)malloc((unsigned) (nregr)*sizeof(double));
  Cb     = (double *)malloc((unsigned) (nregr*nregr)*sizeof(double));
  CbI    = (double *)malloc((unsigned) (nregr*nregr)*sizeof(double));
  xwm    = (double *)malloc((unsigned) (nx)*sizeof(double));


  lambda = (*t).tau / compAVA11((*t).phix,(*t).sigmax2,m);

  (*tprop).phix = (*t).phix;  (*tprop).sigmax2 = (*t).sigmax2;
  (*tprop).phiy = (*t).phiy;  (*tprop).sigmay2 = (*t).sigmay2;
  (*tprop).tau  = (*t).tau;
   for(j=0;j<nregr;j++)(*tprop).beta[j]=(*t).beta[j];

 /* Generate lambda and the coarse level jointly*/
  /* Proposing approximately centered in the previous lambda:*/
  if((lambda+delta)>10.0) ub = 10.0;  else ub = lambda+delta;
  if((lambda-delta)<0.0) lb = 0.0;  else lb = lambda-delta;
  logratio = log(ub - lb);
  prop = runir(lb,ub); 
  if((prop+delta)>10.0) ub = 10.0;  else ub = prop+delta;
  if((prop-delta)<0.0) lb = 0.0;  else lb = prop-delta;
  logratio -= log(ub - lb);
  (*tprop).tau = prop * compAVA11((*t).phix,(*t).sigmax2,m);
  logratio += invgamma_logpdf(prop,0.5*nlambda,0.5*nslambda);
  logratio -= invgamma_logpdf(lambda,0.5*nlambda,0.5*nslambda);
  /* Proposing the coarse level */
  compqygx(tprop,m,nx,ny,Ax,vetaux2,W);
  dpotrf("L",&ny,W,&ny,&info);
  rmvnorm(vetaux2,W,ny,ylevelprop);
  for(j=0;j<ny;j++) vetaux2[j] -= ylevelprop[j];
  logratio += logdet(ny,W) - compexp(ny,vetaux2,W);
  logratio += logpygx(tprop,m,nx,ny,ylevelprop,Ax,&dif2xyprop);
  logratio -= compygt(tprop,m,nx,ny,ylevelprop);
  logratio += logqy(tprop,ny,ylevelprop,&sumy2prop,&sumybyprop);

  compqygx(t,m,nx,ny,Ax,vetaux2,W);
  dpotrf("L",&ny,W,&ny,&info);
  for(j=0;j<ny;j++) vetaux2[j] -= ylevel[j];
  logratio += -logdet(ny,W) + compexp(ny,vetaux2,W);
  logratio -= logpygx(t,m,nx,ny,ylevel,Ax,&dif2xy);
  logratio += compygt(t,m,nx,ny,ylevel);
  logratio -= logqy(t,ny,ylevel,&sumy2,&sumyby);

  if (log(runi())<(logratio)){ 
    (*t).tau = (*tprop).tau;
    lambda = prop;
    inc = 1;
    dcopy(&ny,ylevelprop,&inc,ylevel,&inc);
    sumy2 = sumy2prop;
    sumyby = sumybyprop;
    dif2xy = dif2xyprop;
  }
  else (*tprop).tau = (*t).tau;


 /* Generate phi_y */
  vaux = 1/(sumy2 / (*t).sigmay2 + 1/sphiy);
  maux = vaux*(sumyby / (*t).sigmay2 + mphiy/sphiy);
  do{(*t).phiy = rnor(maux,sqrt(vaux));} 
   while(((*t).phiy >= 1.0) || ((*t).phiy <= -1.0));
  (*tprop).phiy = (*t).phiy; 


 /* Generate sigma_y^2 */
  naux = nsy2 + ny - 1;
  nsaux = nssy2 + sumy2 - 2 * (*t).phiy * sumyby + (*t).phiy * 
            (*t).phiy * (sumy2 + ylevel[0]*ylevel[0] 
              - ylevel[ny-1]*ylevel[ny-1]);
  (*t).sigmay2 = nsaux / rchisq(naux);
  (*tprop).sigmay2 = (*t).sigmay2;


 /* Computing mux */
 for(i=0;i<nx;i++) mux[i]=0;
 for(j=0;j<nregr;j++){
   aux = (*t).beta[j]; auxi = j*nx;
   for(i=0;i<nx;i++) mux[i] += zmatrix[i+auxi] * aux;
 }

 /* Computing xwm = xlevel - mux */
 for(i=0;i<nx;i++) xwm[i]=xlevel[i]-mux[i];

 /* Computing quantities related to xwm */
 sumx2 = 0.0;
 sumxbx = 0.0;
 for(i=1;i<nx;i++){
   sumx2  += xwm[i]*xwm[i];
   sumxbx += xwm[i]*xwm[i-1];
 }


 /* Generate phi_x */
  /* I'm going to use a "quasi" Metropolis step */
  if(((*t).phix+deltaphix)>1.0)  ub =  1.0;  else ub = (*t).phix+deltaphix;
  if(((*t).phix-deltaphix)<-1.0) lb = -1.0;  else lb = (*t).phix-deltaphix;
  logratio = log(ub - lb);
  prop = runir(lb,ub); 
  if((prop+deltaphix)> 1.0) ub =  1.0;  else ub = prop+deltaphix;
  if((prop-deltaphix)<-1.0) lb = -1.0;  else lb = prop-deltaphix;
  logratio -= log(ub - lb);
  (*tprop).phix = prop;
  (*tprop).tau = lambda * compAVA11((*tprop).phix,(*tprop).sigmax2,m);
  logpxgy = compygt(tprop,m,nx,ny,ylevel);
  logpxgyold = compygt(t,m,nx,ny,ylevel);
  logratio += logpxgyold - logpxgy;
  vaux = 1.0/(sumx2 / (*t).sigmax2 + 1/sphix);
  maux = vaux*(sumxbx / (*t).sigmax2 + mphix/sphix);
  aux = prop-maux;
  logratio -= aux*aux*0.5/vaux;
  aux = (*t).phix-maux;
  logratio += aux*aux*0.5/vaux;
  logratio += -0.5*ny*log((*tprop).tau) - 0.5*dif2xy/(*tprop).tau;
  logratio -= -0.5*ny*log((*t).tau) - 0.5*dif2xy/(*t).tau;
  if (log(runi())<logratio){ 
    (*t).phix = prop;  (*t).tau = (*tprop).tau;
    logpxgyold = logpxgy;
  }
  else {(*tprop).phix = (*t).phix;  (*tprop).tau = (*t).tau;}


 /* Generate sigma_x^2 */
  prop = runir((*t).sigmax2/deltasx2,(*t).sigmax2*deltasx2); 
  (*tprop).sigmax2 = prop;
  (*tprop).tau = lambda * compAVA11((*tprop).phix,(*tprop).sigmax2,m);
  logpxgy = compygt(tprop,m,nx,ny,ylevel);
  logratio = logpxgyold - logpxgy;
  logratio += log((*t).sigmax2) - log((*tprop).sigmax2);
  naux = nsx2 + nx - 1 + ny;
  nsaux =  nssx2 + sumx2 - 2 * (*t).phix * sumxbx + (*t).phix * 
            (*t).phix * (sumx2 + xwm[0]*xwm[0] 
              - xwm[nx-1]*xwm[nx-1])
           + dif2xy / (lambda * compAVA11((*tprop).phix,1.0,m));
  logratio += invgamma_logpdf(prop,0.5*naux,0.5*nsaux);
  logratio -= invgamma_logpdf((*t).sigmax2,0.5*naux,0.5*nsaux);
  if (log(runi())<logratio){ 
    (*t).sigmax2 = prop;  (*t).tau = (*tprop).tau;
    logpxgyold = logpxgy; 
  }
  else 
  {(*tprop).sigmax2 = (*t).sigmax2;  (*tprop).tau = (*t).tau;}

  /* Generate beta */
  /* Computing ZtVI = Zmatrix' Vx^{-1}. Note that VxI = Vx^{-1} is a band
     matrix, with non-zero elements belonging to the main diagonal and
     the  subdiagonals. Moreover, the elements of the matrix are:
       VxI[1,1] = VxI[nx,nx] = 1/sigmax2;
       VxI[i,i] = (1+phix^2)/sigmax2,   i=2,...,nx-1;
       VxI[i,i-1] = -phix/sigmax2,  i=2,...,nx;
       VxI[i,i+1] = -phix/sigmax2,  i=1,...,nx-1.  
     Then, we do not need to compute nor store the matrix VxI. 
  */
  /* Computing the first column of ZtVI */
  for(i=0;i<nregr;i++){
   j=nx*i;
   ZtVI[i] = (zmatrix[j] - (*t).phix * zmatrix[1+j])/(*t).sigmax2;
  }
  /* Computing the last column of ZtVI */
  auxi=(nx-1)*nregr;    /* set to the begining of the last column */
  for(i=0;i<nregr;i++){
   j=nx*i;
   ZtVI[i+auxi] = (zmatrix[(nx-1)+j] - (*t).phix * 
                           zmatrix[(nx-2)+j])/(*t).sigmax2;
  }
  /* Computing columns 2,...,(nx-1) of ZtVI */
  for(j=1;j<(nx-1);j++){  
   auxi=j*nregr;      /* set to the begining of the j-th column */
   for(i=0;i<nregr;i++){
    inc=nx*i;
    ZtVI[i+auxi] = ((1+(*t).phix * (*t).phix)*zmatrix[j+inc] - (*t).phix * 
                     (zmatrix[(j-1)+inc]+zmatrix[(j+1)+inc]))/(*t).sigmax2;
   }
  }
  /* Computing ZtVIZ = ZtVI * zmatrix*/
  al=1.0; be=0.0; auxi=nregr;
  auxchar = 'N';
  dgemm("N","N",&auxi,&auxi,&nx,&al,ZtVI,&auxi,zmatrix,&nx,&be,ZtVIZ,&auxi);
  /* Computing CbI = CbetaI + ZtVIZ */
  al=1.0; be=1.0; auxi=nregr;
  dgema("N","N",&auxi,&auxi,&al,CbetaI,&auxi,&be,ZtVIZ,&auxi,CbI,&auxi);
  /* Computing Cb = inv(CbI) */
  inverse(nregr,CbI,Cb);     
  /* Computing auxvet = ZtVI x, that is, assuming mbeta=0 */
  al=1.0; be=0.0; auxi=nregr; inc=1;
  dgemv("N",&auxi,&nx,&al,ZtVI,&auxi,xlevel,&inc,&be,auxvet,&inc);
  /* Computing mb = Cb auxvet */
  al=1.0; be=0.0; auxi=nregr; inc=1;
  dgemv("N",&auxi,&auxi,&al,Cb,&auxi,auxvet,&inc,&be,mb,&inc);
  /* Generating beta  ~ N(mb,Cb) */
  auxi = nregr;
  dpotrf("L",&auxi,Cb,&auxi,&info);
  rmvnorm(mb,Cb,auxi,(*t).beta);

  free(ylevelprop);
  free(vetaux);
  free(vetaux2);
  free(tprop->beta);
  free(tprop);
  free(W);
  free(ZtVI);  
  free(ZtVIZ);
  free(mb);    
  free(auxvet);
  free(Cb);    
  free(CbI);   
  free(mux);

  return;
} /* End of gibbs */


/* 
 * compute the design matrix using seasonal components 
 */

void design_matrix(double *zmatrix, int *harmonics, int nh, int nx)
{
 int i,j,auxi;
 double aux;
 /* Computing the design matrix that models the seasonality */
 for(j=0; j<nh; j++) {
	aux = PI/((double) harmonics[j]);
 	auxi = nx*2*j;
 	for(i=0;i<nx;i++) zmatrix[i+auxi] = cos((i+1)*aux);
 	auxi = nx*(2*j+1);
 	for(i=0;i<nx;i++) zmatrix[i+auxi] = sin((i+1)*aux);
 }
}


/* Computing fine level xlevel => 
    applying the log transformation and removing the mean */

void trans_x(double *xlevel, int nx)
{
 double aux;
 int i;
 for(i=0;i<nx;i++) xlevel[i]=log(xlevel[i]); 
 aux=0.0;
 for(i=0;i<nx;i++) aux+=xlevel[i];
 aux=aux/((double) nx);
 for(i=0;i<nx;i++) xlevel[i] -= aux;
}


/* 
 * MAIN function, used by R
 */

void hrm(
		double *data, int *nx_in, int *m_in, int *Gbig_in,	/* inputs from R */
		int *nys_in, int *nh_in, int *harms_in, 		/* more R inputs */
		double *lambda, double *ysample, double* theta_out	/* outputs to R */
	)
{

 /*FILE *arqin, *arqout;*/
 PARAM *theta, *auxtheta;
 int i, j, Gbig, ny, nx, m, nys, auxi, nh;
 double *xlevel, *ylevel, *Ax, *zmatrix/*, *likelihood, *likepoint*/;
 double sumx2, sumxbx, aux/*, logpxyold, al, be*/;
 /*char entform[3]={'%','d','\0'};*/
 int *harmonics;

 /* set (R) RNG seed -- the argument doesn't matter when RRAND is defined */
 setseed(12345);

 /* COPY INPUTS */
 Gbig =  *Gbig_in;
 m =     *m_in;
 nx =    *nx_in;
 ny =    nx/m;
 nys =   *nys_in;
 nh = *nh_in;
 nregr = 2*nh;

 /* Memory allocation */
 ylevel  = (double *)malloc((unsigned) (ny)*sizeof(double));
 theta   = (PARAM *)malloc((unsigned) (Gbig)*sizeof(PARAM));
 for(i=0; i<Gbig; i++) (*(theta + i)).beta = (double*) malloc(sizeof(double) * nregr);
 Ax      = (double *)malloc((unsigned) (ny)*sizeof(double));
 /*likelihood  = (double *)malloc((unsigned) (Gbig)*sizeof(PARAM));*/
 zmatrix = (double *)malloc((unsigned) (nx*nregr)*sizeof(double));
 harmonics = (int*) malloc(sizeof(int) * nregr);
 for(i=0; i< nh; i++) harmonics[i] = harms_in[i];

 /* Computing fine level xlevel applying the log transformation and removing the mean */
 xlevel = (double*) malloc(sizeof(double) * nx);
 for(i=0; i<nx; i++) xlevel[i] = data[i];
 trans_x(xlevel, nx);

  /* Computing Ax */
 for(j=0;j<ny;j++){
   aux=0;
   for(i=0;i<m;i++) aux+=xlevel[j*m+i];
   Ax[j]=aux/((double)m);
 }

 /* Computing coarse level ylevel 
 */
 for(i=0;i<ny;i++) ylevel[i] = Ax[i];

 /* compute the design matrix using seasonal components */
 design_matrix(zmatrix, harmonics, nh, nx);

 /* Computing some interesting quantities */
 sumx2 = 0.0;
 sumxbx = 0.0;
 for(i=1;i<nx;i++){
   sumx2  += xlevel[i]*xlevel[i];
   sumxbx += xlevel[i]*xlevel[i-1];
 }

 /* Initializing the parameters */
 theta[0].phiy = 0.0;
 theta[0].sigmay2 = 0.1; /* nssy2/rgamma(nsy2); */
 theta[0].phix = 0.0;
 theta[0].sigmax2 =  0.1; /* nssx2/rgamma(nsx2); */
 lambda[0] = 5.0;
 theta[0].tau = lambda[0] * compAVA11((theta[0]).phix,(theta[0]).sigmax2,m);;
 for(i=0;i<nregr;i++) theta[0].beta[i]=0.0;


 /* Generating the Gibbs sample */ 
 for(j=0;j<nys;j++) ysample[j] = ylevel[ny-nys+j];
 auxtheta = theta;
 /*likepoint = likelihood;*/
 /* compqxgt(auxtheta,m,nx,ny,xlevel,zmatrix,likepoint);*/
 for(i=1;i<Gbig;i++){
   if(i % 100 == 0) { 
	   Rprintf("\tGibbs (%c, iteration) = %-4.1f%c, %-8d \n",
			   '%', 100.0 * (double)i/Gbig, '%', i); 
   }
   (*(auxtheta+1)).phiy=(*auxtheta).phiy;
   (*(auxtheta+1)).sigmay2=(*auxtheta).sigmay2;
   (*(auxtheta+1)).phix=(*auxtheta).phix;
   (*(auxtheta+1)).sigmax2=(*auxtheta).sigmax2;
   (*(auxtheta+1)).tau=(*auxtheta).tau;
   for(j=0;j<nregr;j++)(*(auxtheta+1)).beta[j]=(*auxtheta).beta[j];
   auxtheta++;
   /*likepoint++;*/
   gibbs(auxtheta,m,nx,ny,xlevel,ylevel,Ax,sumx2,sumxbx,zmatrix);
   for(j=0;j<nys;j++) ysample[i*nys+j] = ylevel[ny-nys+j];
   lambda[i] = (*auxtheta).tau / compAVA11((*auxtheta).phix,(*auxtheta).sigmax2,m);
 }

 /* Writing the Gibbs sample to a file */
 /*arqout = fopen("riverest_yunknown.dat","w");*/
 auxi = 5+2*nh;
 for(i=0;i<Gbig;i++){
  theta_out[auxi*i + 0] = theta[i].phiy;
  theta_out[auxi*i + 1] = theta[i].sigmay2;
  theta_out[auxi*i + 2] = theta[i].phix;
  theta_out[auxi*i + 3] = theta[i].sigmax2;
  theta_out[auxi*i + 4] = theta[i].tau;
  for(j=0; j<2*nh; j++) theta_out[auxi*i+5+j] = theta[i].beta[j];
 }
 /*fclose(arqout);*/

 /* Releasing memory */
 for(i=0; i<Gbig; i++) free((*(theta + i)).beta);
 free(theta);
 /*free(ysample);*/
 free(xlevel);
 free(ylevel);
 free(Ax);
 /*free(lambda);*/
 /*free(likelihood);*/
 free(zmatrix);
 free(harmonics);

 /* return the modified RNG seed to R */
 putseed();

} /* End of main */
