#include "setup.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "rand.h"
#include "rmvnormd.h"
#include "matrix.h"
#include "tipos.h"

int nregrf;

/* Compute forecasts for the last year observed, assuming observations
   only till december of 1989 */

/* # of years for prediction */


typedef struct { double *lag/*[mm*window]*/; } VETSAI;


void foreAR1(PARAM *auxtheta, double xlast,
             double *mAR1, double *CAR1, int m)
{
  int i,j,auxi;
  double aux, aux2;

  /* Computing mAR1 without coarse information */
  aux2 = 1.0;
  for(i=0;i<m;i++){
    aux2 *=  (*auxtheta).phix;
    mAR1[i] = xlast*aux2;
  }

  /* Computing CAR1 without coarse information */
  /* Diagonal: */
  aux = 1.0;
  aux2 = 1.0;
  for(j=0;j<m;j++){
    CAR1[j+j*m] = (*auxtheta).sigmax2 * aux;
    aux2 *= (*auxtheta).phix * (*auxtheta).phix;
    aux += aux2;
  }
  /* Lower part */
  for(j=0;j<m-1;j++){
    aux = CAR1[j+j*m];
    auxi = j*m;
    for(i=j+1;i<m;i++){
      aux *= (*auxtheta).phix;
      CAR1[i+auxi] = aux;
    }
  }
  /* Upper part */
  for(j=1;j<m;j++){
    auxi = j*m;
    for(i=0;i<j;i++) CAR1[i+auxi] = CAR1[j+i*m];
  }

} /* End of foreAR1 */

void compforecoarse(PARAM *theta, double *ylevel, double xlast, 
                    double *mAR1, double *CAR1, int m, int nx, int ny)
{
 double Vyny, muy;
 double *W, *W11, *W12, *W21, W22, *Vy;
 double *caux, *vaux, *vaux2;
 int i, j, auxi, info, inc;
 double auxd, alpha, beta;
 double m1, m2, V1, V2;

 W = (double *)malloc((unsigned) (ny*ny)*sizeof(double));
 W11 = (double *)malloc((unsigned) ((ny-1)*(ny-1))*sizeof(double));
 W12 = (double *)malloc((unsigned) (ny-1)*sizeof(double));
 W21 = (double *)malloc((unsigned) (ny-1)*sizeof(double));
 Vy = (double *)malloc((unsigned) (ny*ny)*sizeof(double));
 caux = (double *)malloc((unsigned) (m*m)*sizeof(double));
 vaux = (double *)malloc((unsigned) (m)*sizeof(double));
 vaux2 = (double *)malloc((unsigned) (m)*sizeof(double));

 compW(theta,m,nx,ny,W);

 /* Get W11, W12, W22 */
 i = ny - 1;
 alpha=1.0;
 dgemt("N",&i,&i,&alpha,W,&ny,W11,&i);

 for(i=0;i<(ny-1);i++) W12[i] = W[(ny-1)*ny+i];

 for(i=0;i<(ny-1);i++) W21[i] = W12[i];

 W22 = W[ny*ny-1];

 /* Compute V1 = W22 - W21 W11^{-1} W12 */
 i = ny - 1;
 j=1;
 dposv("L",&i,&j,W11,&i,W12,&i,&info);     /* W12 = W11^{-1} W12 */
 V1 = W22;
 for(i=0;i<(ny-1);i++) V1 -= W21[i]*W12[i];

 /* Compute m1 = W21 W11^{-1} y[1:(t-1)] */
 m1 = 0.0;
 for(i=0;i<(ny-1);i++) m1 += W12[i] * ylevel[i];

 /* Compute m2 and V2 */
 inverse(m,CAR1,caux);      /* caux = CAR1^{-1} */
 alpha=1.0, beta=0.0; inc=1;
 /* vaux2 = caux * mAR1 */
 dgemv("N",&m,&m,&alpha,caux,&m,mAR1,&inc,&beta,vaux2,&inc); 
 /* caux = (caux + A' A /(tau*m*m))^{-1} */
 auxd = 1.0 / ((*theta).tau*m*m);
 auxi = m*m;
 for(i=0;i<auxi;i++) caux[i] += auxd; 
 inverse(m,caux,caux);
 /* vaux = caux * vaux2 */
 alpha=1.0, beta=0.0; inc=1;
 dgemv("N",&m,&m,&alpha,caux,&m,vaux2,&inc,&beta,vaux,&inc); 
 /* m2 = A vaux / tau */
 m2=0.0;
 for(i=0;i<m;i++) m2 += vaux[i];
 m2 /= ((*theta).tau*m);
 /* V2 = (A caux A' / tau^2)^{-1} */
 V2 = 0.0;
 auxi = m*m;
 for(i=0;i<auxi;i++) V2 += caux[i];
 V2 = (*theta).tau * (*theta).tau * m * m / V2;
 /* m2 = m2 * V2 */
 m2 *= (-1.0) * V2;

 /* Compute Vyny */
 Vyny = 1.0 / (1.0 / (*theta).sigmay2 + 1.0 / (*theta).tau -1.0/V1 - 1.0/V2);

 /* Compute muy */
 muy = Vyny*((*theta).phiy * ylevel[ny-2] / (*theta).sigmay2 - m1/V1 - m2/V2);

 /* Generate an observation from the predictive distribution */
 ylevel[ny-1] = rnor(muy,sqrt(Vyny));

 /* Isto era so para verificar as previsoes 
 ylevel[ny-1] = muy;
 */

 free(W);
 free(W11);
 free(W12);
 free(W21);
 free(Vy);
 free(caux);
 free(vaux);
 free(vaux2);

}   /* End of compforecoarse */



void compforefine(PARAM *auxtheta, double yfore, double xlast, double
		  *mux, double *mAR1, double *CAR1, double *forecast, int m)
{
 double *foremean, *forevar, *auxvet;
 int i,j,auxi;
 double aux/*,aux2*/;
 int inc, /*rows, cols,*/ info;
 double al, be;

 foremean = (double *)malloc((unsigned) (m)*sizeof(double));
 forevar  = (double *)malloc((unsigned) (m*m)*sizeof(double));
 auxvet   = (double *)malloc((unsigned) (m)*sizeof(double));

  /* foremean = mAR1 */
  for(j=0;j<m;j++) foremean[j] = mAR1[j];

  /* forevar = CAR1 */
  auxi=m*m;
  for(j=0;j<auxi;j++) forevar[j] = CAR1[j];

  /* Invert forevar */
  inverse(m,forevar,forevar);

  /* foremean = forevar * foremean */
  al = 1.0;  be = 0.0;   inc = 1; auxi = m;
  for(i=0;i<m;i++) auxvet[i] = foremean[i];
  dgemv("N",&auxi,&auxi,&al,forevar,&auxi,auxvet,&inc,&be,foremean,&inc);

  /* forevar = forevar + A A' /tau */
  aux = 1.0/(m*m * (*auxtheta).tau);
  for(j=0;j<m;j++){
    auxi = j*m;
    for(i=0;i<m;i++) forevar[i+auxi] += aux;
  }

  /* Invert forevar */
  inverse(m,forevar,forevar);

  /* foremean = foremean + A y[ny] /tau */
  aux = yfore / (m * (*auxtheta).tau);
  for(i=0;i<m;i++) foremean[i] += aux;
  
  /* foremean = forevar * foremean */
  al = 1.0;  be = 0.0;   inc = 1; auxi = m;
  for(i=0;i<m;i++) auxvet[i] = foremean[i];
  dgemv("N",&auxi,&auxi,&al,forevar,&auxi,auxvet,&inc,&be,foremean,&inc);

  for(i=0;i<m;i++) foremean[i] += mux[i];

  /* Generate x[(nx+1):(nx+m)] ~ N(foremean, forevar) */
  auxi = m;
  dpotrf("L",&auxi,forevar,&auxi,&info);
  rmvnorm(foremean,forevar,auxi,forecast);

 /* Isto era so para verificar as previsoes 
  for(i=0;i<m;i++) forecast[i] = foremean[i];
 */

  free(foremean); 
  free(forevar);  
  free(auxvet);   

}   /* End of compforefine */


void hrm_fcast(
	int *window_in, double *data, int *nx_in, int *m_in,int *consid_in, 
	int *nys_in, int *nh_in, int *harms_in, double *ysample, double* theta_in,
	double *sample_out
	)
{

 /*FILE *arqin, *arqout;*/
 int i, j, g, consid;
 double *xlevel, *ylevel, *mux;
 PARAM *theta, *auxtheta;
 double *zmatrix;
 double *mAR1, *CAR1;
 VETSAI *forecast, *auxfore;
 int nx, ny, year, auxi;
 int m;
 double xlast, aux;
 int nys, nh, window;
 int *harmonics;

 /* COPY INPUTS */
 consid =  *consid_in; 
 m =     *m_in; 
 nx =    *nx_in;
 ny =    nx/m;
 nys =   *nys_in;
 nh = *nh_in;
 nregrf = 2*nh;
 window = *window_in;

 ylevel  = (double *)malloc((unsigned) (ny+window)*sizeof(double));
 /*xlevel  = (double *)malloc((unsigned) (nx)*sizeof(double));*/
 zmatrix = (double *)malloc((unsigned) ((nx+m*window)*nregrf)*sizeof(double));
 mux     = (double *)malloc((unsigned) (nx+m*window)*sizeof(double));
 theta = (PARAM *)malloc((unsigned) (consid)*sizeof(PARAM));
 for(i=0; i<consid; i++) (*(theta + i)).beta = (double*) malloc(sizeof(double) * nregrf);
 forecast = (VETSAI *)malloc((unsigned) (consid)*sizeof(VETSAI));
 for(i=0; i<consid; i++) (*(forecast + i)).lag = (double*) malloc(sizeof(double) * m * window);
 mAR1 = (double *)malloc((unsigned) (m)*sizeof(double));
 CAR1 = (double *)malloc((unsigned) (m*m)*sizeof(double));
 harmonics = (int*) malloc(sizeof(int) * nregrf);
 for(i=0; i< nh; i++) harmonics[i] = harms_in[i];

 /* Computing fine level xlevel applying the log transformation and removing the mean */
 xlevel = (double*) malloc(sizeof(double) * nx);
 for(i=0; i<nx; i++) xlevel[i] = data[i];
 trans_x(xlevel, nx);

 design_matrix(zmatrix, harmonics, nh, nx+m*window);

 /* get theta data structure from input */
 auxi = 5+2*nh;
 for(i=0;i<consid;i++){
  theta[i].phiy = theta_in[auxi*i + 0];
  theta[i].sigmay2 = theta_in[auxi*i + 1];
  theta[i].phix = theta_in[auxi*i + 2];
  theta[i].sigmax2 =theta_in[auxi*i + 3];
  theta[i].tau = theta_in[auxi*i + 4];
  for(j=0; j<2*nh; j++) theta[i].beta[j] = theta_in[auxi*i+5+j];
 }

 /* init mux and ylevel */
 for(i=0;i<(nx+m*window);i++) mux[i]=0.0;
 for(i=0;i<(ny-nys);i++) ylevel[i]=0.0;

 for(i=0;i<consid;i++) for(j=0;j<(m*window);j++) sample_out[m*window*i + j] = 0.0;

 auxtheta = theta; 
 auxfore  = forecast;
 for(g=0;g<consid;g++)
 {
  if(g % 100 == 0) { 
    Rprintf("\tGibbs (%c, iteration) = %-4.1f%c, %-8d \n",
                    '%', 100.0 * (double)g/consid, '%', g); 
  }

  /* Compute mux = Z beta */
  for(i=0;i<(nx+m*window);i++) mux[i]=0;
  for(j=0;j<nregrf;j++){
    aux = (*auxtheta).beta[j];
    auxi = j*(nx+m*window);
    for(i=0;i<(nx+m*window);i++) 
      mux[i] += zmatrix[i+auxi] * aux;
  }

  xlast = xlevel[nx-1]-mux[nx-1];

  for(i=(ny-nys);i<ny;i++) ylevel[i]=ysample[g*nys+i-(ny-nys)];

  for(year=0;year<window;year++){
    foreAR1(auxtheta,xlast,mAR1,CAR1,m);

    /* Generate y[ny] */
    compforecoarse(auxtheta,ylevel,xlast,mAR1,CAR1,m,nx+m*(year+1),ny+year+1);

    /* Generating the forecasts */
    compforefine(auxtheta,ylevel[ny+year],xlast,mux+nx+m*year,mAR1,CAR1,(*auxfore).lag+m*year,m);

    xlast = (*auxfore).lag[m*(year+1)-1]-mux[nx+m*(year+1)-1];

  }
  /* Move on to the next gibbs observation */
  auxtheta++;
  auxfore++;
 }

 /* Writing the forecast sample to a text file */
 auxfore = forecast;
 for(i=0;i<consid;i++){
  for(j=0;j<(m*window);j++) sample_out[m*window*i + j] = (*auxfore).lag[j];
  auxfore++;
 }

 free(ylevel);
 free(mux);
 for(i=0; i<consid; i++) free((*(theta + i)).beta);
 free(theta);
 for(i=0; i<consid; i++) free((*(forecast + i)).lag);
 free(forecast);
 free(mAR1);
 free(CAR1);
 free(zmatrix);
}
