#include <stdlib.h>	
#include <assert.h>	
#include <stdio.h>	
#include <math.h>	
#include "matrix.h"
#include "tipos.h"

int  inverse(int n, double *a_in, double *v_out)
/* returns in v the inverse of a, i.e. v = a^-1
   for a real symmetric pos def */
{ 
  int 
    infoc, info, i, j;
  double al;
  char
    uplo='U';
 
  al = 1.0;

  if (a_in != v_out) dgemt("N",&n,&n,&al,a_in,&n,v_out,&n);
  dpotrf_(&uplo, &n, v_out, &n, &infoc); /* chol decomp */
  dpotri_(&uplo,&n,v_out,&n,&info);
  for(i=0;i<n;i++){
    for(j=0;j<=i;j++){
      v_out[j*n+i] = v_out[i*n+j];
    }
  }

  if ((info != 0)|(infoc != 0)) 
    printf("\n Warning: spurious matrix inversion.\n");
  return(info);
}


int  inverseg(int n, double *a_in, int lda, double *v_out, int ldv)
/* returns in v the inverse of a, i.e. v = a^-1
   for a real symmetric pos def */
{ 
  int 
    infoc, info, i, j;
  double al;
  char
    uplo='U';
 
  al = 1.0;

  if (a_in != v_out) dgemt("N",&n,&n,&al,a_in,&lda,v_out,&ldv);
  dpotrf_(&uplo, &n, v_out, &ldv, &infoc); /* chol decomp */
  dpotri_(&uplo,&n,v_out,&ldv,&info);

  for(i=0;i<n;i++){
    for(j=0;j<=i;j++){
      v_out[j*ldv+i] = v_out[i*ldv+j];
    }
  }

  if ((info != 0)|(infoc != 0)) 
    printf("\n Warning: spurious matrix inversion.\n");
  return(info);
}

void cholesky(int nx, double *D, double *Lauxx)
{
int info;
double alpha;

  /* Computing Lauxx = Cholesky(D) */
  alpha = 1.0;
  if (D != Lauxx) dgemt("N",&nx,&nx,&alpha,D,&nx,Lauxx,&nx); 
  dpotrf_("L",&nx,Lauxx,&nx,&info);
  return;

} /* End of Cholesky */


void printmatrix(int m, int n, double *matrix)
{
  int i, j;

  for(i=0;i<m;i++){
    for(j=0;j<n;j++) printf("%f ",matrix[i+j*m]);
    printf("\n");
  }

} /* End of printmatrix */

double dnrsq(int *n, double *x, int *incx)
{
  int i;
  double aux;
  if(*incx != 1) printf("Warning: in function dnrsq, incx != 1\n");
  aux =0.0;  for(i=0;i<(*n);i++) aux += x[i]*x[i];
  return aux;
}

int dgemt(char *trans, int *m, int *n, double *alpha, double *a, int *lda, double *b, int *ldb)
{
  /* In this implementation, lda and ldb are ignored. */
  int i, j, auxi;
  auxi = (*m)*(*n);
  if((*trans == 'N')||(*trans == 'n'))
    for(i=0;i<auxi;i++) b[i] = *alpha * a[i];
  else
  {
    if((*trans == 'T')||(*trans == 't')) 
      for(i=0;i<(*m);i++) 
      {
	auxi = (*n) * i;
        for(j=0;j<(*n);j++) b[(*m) * j + i] = a[auxi + j];
      }
    else printf("Warning: in function dgemt, invalid transformation");
  }
  return 0;
}

int dgema(char *transa, char *transb,  int *m, int *n, double *alpha, double *a, int *lda, double *beta, double *b, int *ldb, double *c, int *ldc)
{
 /* In this implementation, transa, transb, lda, ldb, ldc are ignored. */
 int i, auxi;
 auxi = (*m)*(*n);
 if(((*transa == 'N')||(*transa == 'n'))&&((*transb == 'N')||(*transb == 'n')))
  for(i=0;i<auxi;i++) c[i]=(*alpha) * a[i] + (*beta) * b[i];
 else 
  printf("Warning: in function dgema, invalid transformation");
 return 0;
}
